# SPDX-FileCopyrightText: 2015 Eric Larson
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations


from datetime import datetime, timezone
from typing import TYPE_CHECKING

from pip._vendor.cachecontrol.cache import BaseCache

if TYPE_CHECKING:
    from redis import Redis


class RedisCache(BaseCache):
    def __init__(self, conn: Redis[bytes]) -> None:
        self.conn = conn

    def get(self, key: str) -> bytes | None:
        return self.conn.get(key)

    def set(
        self, key: str, value: bytes, expires: int | datetime | None = None
    ) -> None:
        if not expires:
            self.conn.set(key, value)
        elif isinstance(expires, datetime):
            now_utc = datetime.now(timezone.utc)
            if expires.tzinfo is None:
                now_utc = now_utc.replace(tzinfo=None)
            delta = expires - now_utc
            self.conn.setex(key, int(delta.total_seconds()), value)
        else:
            self.conn.setex(key, expires, value)

    def delete(self, key: str) -> None:
        self.conn.delete(key)

    def clear(self) -> None:
        """Helper for clearing all the keys in a database. Use with
        caution!"""
        for key in self.conn.keys():
            self.conn.delete(key)

    def close(self) -> None:
        """Redis uses connection pooling, no need to close the connection."""
        pass



import asyncio
import json
from pathlib import Path
from types import SimpleNamespace

import pytest

import master


@pytest.fixture(autouse=True)
def restore_manager():
    """每个用例后还原全局 MANAGER，避免互相影响。"""

    original = master.MANAGER
    yield
    master.MANAGER = original


class DummyBot:
    """简化版 Bot，用于记录发送的消息。"""

    def __init__(self) -> None:
        self.messages = []

    async def send_message(self, chat_id: int, text: str, **kwargs) -> None:
        self.messages.append((chat_id, text, kwargs))


class DummyMessage:
    """模拟 aiogram Message，仅保留测试所需接口。"""

    def __init__(self, chat_id: int) -> None:
        self.chat = SimpleNamespace(id=chat_id)
        self.from_user = SimpleNamespace(id=chat_id, username=None)
        self.text = "/upgrade"
        self.replies = []

    async def answer(self, text: str, **kwargs) -> None:
        self.replies.append((text, kwargs))


@pytest.fixture
def update_state_path(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> Path:
    """为每个用例隔离 update_state.json 位置。"""

    state_path = tmp_path / "update_state.json"
    state_path.parent.mkdir(parents=True, exist_ok=True)
    monkeypatch.setattr(master, "UPDATE_STATE_PATH", state_path)
    return state_path


@pytest.mark.asyncio
async def test_ensure_update_state_without_latest(monkeypatch: pytest.MonkeyPatch, update_state_path: Path):
    """无可用版本时仅记录 last_check。"""

    async def fake_fetch():
        return None

    monkeypatch.setattr(master, "_fetch_latest_version", fake_fetch)
    state = await master._ensure_update_state(force=True)
    assert "last_check" in state
    assert "latest_version" not in state
    # 确保状态已写入文件
    written = json.loads(update_state_path.read_text(encoding="utf-8"))
    assert "last_check" in written


@pytest.mark.asyncio
async def test_ensure_update_state_with_new_version(monkeypatch: pytest.MonkeyPatch, update_state_path: Path):
    """检测到新版本时重置已通知列表。"""

    update_state_path.write_text(
        json.dumps({"latest_version": "1.0.19", "notified_chat_ids": [1, 2]}, ensure_ascii=False),
        encoding="utf-8",
    )

    async def fake_fetch():
        return "9.9.9"

    monkeypatch.setattr(master, "_fetch_latest_version", fake_fetch)
    state = await master._ensure_update_state(force=True)
    assert state["latest_version"] == "9.9.9"
    assert state["notified_chat_ids"] == []


@pytest.mark.asyncio
async def test_maybe_notify_update_single_chat(monkeypatch: pytest.MonkeyPatch, update_state_path: Path):
    """同一 chat 仅提醒一次。"""

    state = {
        "latest_version": "9.9.9",
        "notified_chat_ids": [],
        "last_check": master._utcnow().isoformat(),
    }
    update_state_path.write_text(json.dumps(state, ensure_ascii=False), encoding="utf-8")

    async def fake_fetch():
        return "9.9.9"

    monkeypatch.setattr(master, "_fetch_latest_version", fake_fetch)
    bot = DummyBot()
    notified = await master._maybe_notify_update(bot, chat_id=100, force_check=False)
    assert notified is True
    assert len(bot.messages) == 1

    notified_again = await master._maybe_notify_update(bot, chat_id=100, force_check=False)
    assert notified_again is False
    assert len(bot.messages) == 1


@pytest.mark.asyncio
async def test_maybe_notify_update_multiple_chats(monkeypatch: pytest.MonkeyPatch, update_state_path: Path):
    """不同 chat 均会收到同一版本的提醒。"""

    state = {
        "latest_version": "8.0.0",
        "notified_chat_ids": [],
        "last_check": master._utcnow().isoformat(),
    }
    update_state_path.write_text(json.dumps(state, ensure_ascii=False), encoding="utf-8")

    async def fake_fetch():
        return "8.0.0"

    monkeypatch.setattr(master, "_fetch_latest_version", fake_fetch)
    bot = DummyBot()
    await master._maybe_notify_update(bot, chat_id=1, force_check=False)
    await master._maybe_notify_update(bot, chat_id=2, force_check=False)
    assert {chat_id for chat_id, *_ in bot.messages} == {1, 2}


@pytest.mark.asyncio
async def test_maybe_notify_update_skips_old_version(update_state_path: Path):
    """当前版本不落后时不提醒。"""

    state = {
        "latest_version": master.__version__,
        "notified_chat_ids": [],
    }
    update_state_path.write_text(json.dumps(state, ensure_ascii=False), encoding="utf-8")
    bot = DummyBot()
    notified = await master._maybe_notify_update(bot, chat_id=1, state=state)
    assert notified is False
    assert bot.messages == []


@pytest.mark.asyncio
async def test_notify_update_to_targets(monkeypatch: pytest.MonkeyPatch, update_state_path: Path):
    """批量通知会遍历所有目标。"""

    state = {
        "latest_version": "7.0.0",
        "notified_chat_ids": [],
        "last_check": master._utcnow().isoformat(),
    }
    update_state_path.write_text(json.dumps(state, ensure_ascii=False), encoding="utf-8")

    async def fake_fetch():
        return "7.0.0"

    monkeypatch.setattr(master, "_fetch_latest_version", fake_fetch)
    bot = DummyBot()
    await master._notify_update_to_targets(bot, [11, 22], force_check=False)
    assert len(bot.messages) == 2


def test_trigger_upgrade_pipeline_success(monkeypatch: pytest.MonkeyPatch):
    """成功触发升级命令时返回 True。"""

    recorded = {}

    class DummyPopen:
        def __init__(self, args, cwd, stdout, stderr):
            recorded["args"] = args
            recorded["cwd"] = cwd

    monkeypatch.setattr(master.subprocess, "Popen", DummyPopen)
    ok, err = master._trigger_upgrade_pipeline()
    assert ok is True
    assert err is None
    assert recorded["args"][2] == "pipx upgrade vibego && vibego stop && vibego start"


def test_trigger_upgrade_pipeline_failure(monkeypatch: pytest.MonkeyPatch):
    """Popen 抛错时返回 False 并包含错误信息。"""

    def raising(*_args, **_kwargs):
        raise OSError("boom")

    monkeypatch.setattr(master.subprocess, "Popen", raising)
    ok, err = master._trigger_upgrade_pipeline()
    assert ok is False
    assert "boom" in err


@pytest.mark.asyncio
async def test_cmd_upgrade_authorized(monkeypatch: pytest.MonkeyPatch):
    """授权用户执行 /upgrade 时发送提示并触发命令。"""

    message = DummyMessage(chat_id=999)
    monkeypatch.setattr(master, "_trigger_upgrade_pipeline", lambda: (True, None))
    master.MANAGER = SimpleNamespace(is_authorized=lambda _: True)
    await master.cmd_upgrade(message)
    assert message.replies, "应至少回复一条消息"
    assert "pipx upgrade vibego" in message.replies[0][0]


@pytest.mark.asyncio
async def test_cmd_upgrade_unauthorized(monkeypatch: pytest.MonkeyPatch):
    """未授权用户无法执行 /upgrade。"""

    message = DummyMessage(chat_id=321)
    master.MANAGER = SimpleNamespace(is_authorized=lambda _: False)
    await master.cmd_upgrade(message)
    assert message.replies[0][0] == "未授权。"



"""
测试 master 重启功能的测试用例
包括：
1. 重启按钮不应该刷新项目列表
2. 重启请求应该被正确处理
3. 其他按钮仍然应该刷新项目列表
"""
import asyncio
import json
from datetime import datetime
from pathlib import Path
from types import SimpleNamespace
from unittest.mock import AsyncMock, patch

import pytest

import master
from project_repository import ProjectRepository


@pytest.fixture(autouse=True)
def reset_state():
    """重置全局状态"""
    master.PROJECT_WIZARD_SESSIONS.clear()
    master.PROJECT_WIZARD_LOCK = asyncio.Lock()
    yield
    master.PROJECT_WIZARD_SESSIONS.clear()
    master.PROJECT_WIZARD_LOCK = asyncio.Lock()
    master.PROJECT_REPOSITORY = None
    master.MANAGER = None


@pytest.fixture
def repo(tmp_path: Path, monkeypatch) -> ProjectRepository:
    """创建测试用的项目仓库"""
    config_dir = tmp_path / "config"
    config_dir.mkdir()
    json_path = config_dir / "projects.json"
    initial = [
        {
            "bot_name": "TestBot",
            "bot_token": "123456:ABCDEFGHIJKLMNOPQRSTUVWXYZ012345",
            "project_slug": "test",
            "default_model": "codex",
            "workdir": str(tmp_path),
            "allowed_chat_id": 100,
        }
    ]
    json_path.write_text(json.dumps(initial, ensure_ascii=False, indent=2), encoding="utf-8")
    db_path = config_dir / "master.db"
    repository = ProjectRepository(db_path, json_path)
    master.PROJECT_REPOSITORY = repository
    monkeypatch.setenv("MASTER_ADMIN_IDS", "1")
    return repository


def _build_manager(repo: ProjectRepository, tmp_path: Path) -> master.MasterManager:
    """构建 MasterManager 实例"""
    records = repo.list_projects()
    configs = [master.ProjectConfig.from_dict(record.to_dict()) for record in records]
    state_path = tmp_path / "state.json"
    state_store = master.StateStore(state_path, {cfg.project_slug: cfg for cfg in configs})
    return master.MasterManager(configs, state_store=state_store)


class DummyMessage:
    """模拟 Telegram Message 对象"""
    def __init__(self, chat_id: int = 1) -> None:
        self.text = ""
        self.chat = SimpleNamespace(id=chat_id)
        self.from_user = SimpleNamespace(id=chat_id, username="tester")
        self.message_id = 1
        self.bot = AsyncMock()
        self._answers = []
        self._edits = []

    async def answer(self, text: str, **kwargs):
        self._answers.append((text, kwargs))

    async def edit_text(self, text: str, **kwargs):
        self._edits.append((text, kwargs))

    async def edit_reply_markup(self, **kwargs):
        self._edits.append(("reply_markup", kwargs))


class DummyCallback:
    """模拟 Telegram CallbackQuery 对象"""
    def __init__(self, data: str, chat_id: int = 1, message: DummyMessage | None = None) -> None:
        self.data = data
        self.from_user = SimpleNamespace(id=chat_id, username="tester")
        self.message = message or DummyMessage(chat_id)
        self._answers = []

    async def answer(self, text: str | None = None, show_alert: bool = False):
        self._answers.append((text, show_alert))


def test_restart_master_does_not_refresh_project_list(repo: ProjectRepository, tmp_path: Path, monkeypatch):
    """
    测试用例 1：点击"重启 Master"按钮后，不应该刷新项目列表

    验证点：
    - 重启请求被正确处理
    - 消息没有被编辑（不刷新项目列表）
    - 回调答复正确发送
    """
    manager = _build_manager(repo, tmp_path)
    master.MANAGER = manager

    # 创建启动脚本（避免重启失败）
    start_script = tmp_path / "scripts" / "start.sh"
    start_script.parent.mkdir(parents=True, exist_ok=True)
    start_script.write_text("#!/bin/bash\necho 'mock start'", encoding="utf-8")
    start_script.chmod(0o755)

    # 设置 ROOT_DIR 以便找到启动脚本
    monkeypatch.setattr(master, "ROOT_DIR", tmp_path)

    # 模拟重启信号写入
    signal_path = tmp_path / "state" / "restart_signal.json"
    signal_path.parent.mkdir(parents=True, exist_ok=True)
    monkeypatch.setattr(master, "RESTART_SIGNAL_PATH", signal_path)

    # 创建回调
    callback_message = DummyMessage()
    callback = DummyCallback("project:restart_master:*", message=callback_message)

    # 执行回调处理
    async def _invoke():
        # Mock _process_restart_request 以避免实际重启
        original_process = master._process_restart_request

        async def mock_restart(message, *, trigger_user=None, manager=None):
            # 记录重启被调用
            message._restart_called = True
            await message.answer("已收到重启指令，运行期间 master 会短暂离线，重启后所有 worker 需稍后手动启动。")

        with patch.object(master, '_process_restart_request', new=mock_restart):
            # 导入 FSMContext 以便调用处理函数
            from aiogram.fsm.context import FSMContext
            from aiogram.fsm.storage.base import StorageKey
            from aiogram.fsm.storage.memory import MemoryStorage

            storage = MemoryStorage()
            key = StorageKey(bot_id=0, chat_id=1, user_id=1)
            fsm_state = FSMContext(storage=storage, key=key)

            await master.on_project_action(callback, fsm_state)

    asyncio.run(_invoke())

    # 验证：回调应该返回"已收到重启指令"
    assert callback._answers, "应该有回调答复"
    assert callback._answers[0][0] == "已收到重启指令", "回调答复内容应该正确"

    # 验证：消息不应该被编辑（不刷新项目列表）
    assert len(callback_message._edits) == 0, "重启时不应该编辑消息（不刷新项目列表）"

    # 验证：重启请求应该被调用
    assert hasattr(callback_message, '_restart_called'), "重启请求应该被调用"


def test_other_actions_still_refresh_project_list(repo: ProjectRepository, tmp_path: Path, monkeypatch):
    """
    测试用例 2：其他按钮操作（如停止项目）仍然应该刷新项目列表

    验证点：
    - 停止项目请求被正确处理
    - 消息被编辑（刷新项目列表）
    """
    manager = _build_manager(repo, tmp_path)
    master.MANAGER = manager
    master.PROJECT_REPOSITORY = repo

    # 设置项目状态为运行中
    manager.state_store.update("test", model="codex", status="running")

    # Mock stop_worker
    async def mock_stop_worker(cfg, *, update_state=True):
        manager.state_store.update(cfg.project_slug, status="stopped")

    monkeypatch.setattr(manager, "stop_worker", AsyncMock(side_effect=mock_stop_worker))

    # 创建回调
    callback_message = DummyMessage()
    callback = DummyCallback("project:stop:test", message=callback_message)

    async def _invoke():
        from aiogram.fsm.context import FSMContext
        from aiogram.fsm.storage.base import StorageKey
        from aiogram.fsm.storage.memory import MemoryStorage

        storage = MemoryStorage()
        key = StorageKey(bot_id=0, chat_id=1, user_id=1)
        fsm_state = FSMContext(storage=storage, key=key)

        await master.on_project_action(callback, fsm_state)

    asyncio.run(_invoke())

    # 验证：消息应该被编辑（刷新项目列表）
    assert len(callback_message._edits) > 0, "停止项目后应该编辑消息（刷新项目列表）"

    # 验证：编辑的内容应该是项目概览
    text, kwargs = callback_message._edits[0]
    assert text == "请选择操作：", "应该刷新项目概览"


def test_restart_master_without_message_object(repo: ProjectRepository, tmp_path: Path):
    """
    测试用例 3：边界场景 - 重启按钮回调缺少 message 对象

    验证点：
    - 应该记录错误日志
    - 应该立即返回，不执行重启
    """
    manager = _build_manager(repo, tmp_path)
    master.MANAGER = manager

    # 创建没有 message 的回调
    callback = DummyCallback("project:restart_master:*")
    callback.message = None

    async def _invoke():
        from aiogram.fsm.context import FSMContext
        from aiogram.fsm.storage.base import StorageKey
        from aiogram.fsm.storage.memory import MemoryStorage

        storage = MemoryStorage()
        key = StorageKey(bot_id=0, chat_id=1, user_id=1)
        fsm_state = FSMContext(storage=storage, key=key)

        await master.on_project_action(callback, fsm_state)

    # 应该不抛出异常
    asyncio.run(_invoke())

    # 验证：回调应该有答复
    assert callback._answers, "应该有回调答复"


def test_restart_master_with_unauthorized_user(repo: ProjectRepository, tmp_path: Path, monkeypatch):
    """
    测试用例 4：异常场景 - 未授权用户尝试重启

    验证点：
    - 应该拒绝未授权用户的重启请求
    - 应该发送"未授权"消息
    """
    manager = _build_manager(repo, tmp_path)
    master.MANAGER = manager

    # 设置授权用户为 999，而回调用户为 1
    monkeypatch.setenv("MASTER_ADMIN_IDS", "999")

    # 创建启动脚本
    start_script = tmp_path / "scripts" / "start.sh"
    start_script.parent.mkdir(parents=True, exist_ok=True)
    start_script.write_text("#!/bin/bash\necho 'mock start'", encoding="utf-8")
    start_script.chmod(0o755)

    monkeypatch.setattr(master, "ROOT_DIR", tmp_path)

    # 创建回调（用户 ID 为 1）
    callback_message = DummyMessage(chat_id=1)
    callback = DummyCallback("project:restart_master:*", chat_id=1, message=callback_message)

    async def _invoke():
        from aiogram.fsm.context import FSMContext
        from aiogram.fsm.storage.base import StorageKey
        from aiogram.fsm.storage.memory import MemoryStorage

        storage = MemoryStorage()
        key = StorageKey(bot_id=0, chat_id=1, user_id=1)
        fsm_state = FSMContext(storage=storage, key=key)

        await master.on_project_action(callback, fsm_state)

    asyncio.run(_invoke())

    # 验证：应该没有调用重启
    # 由于未授权，回调处理应该在早期就返回


def test_stop_all_refreshes_project_list(repo: ProjectRepository, tmp_path: Path, monkeypatch):
    """
    测试用例 5：验证"停止全部项目"按钮会刷新项目列表

    验证点：
    - 停止全部项目请求被正确处理
    - 消息被编辑（刷新项目列表）
    """
    manager = _build_manager(repo, tmp_path)
    master.MANAGER = manager
    master.PROJECT_REPOSITORY = repo

    # Mock stop_all
    async def mock_stop_all(*, update_state=True):
        for cfg in manager.configs:
            manager.state_store.update(cfg.project_slug, status="stopped")

    monkeypatch.setattr(manager, "stop_all", AsyncMock(side_effect=mock_stop_all))

    # 创建回调
    callback_message = DummyMessage()
    callback = DummyCallback("project:stop_all:*", message=callback_message)

    async def _invoke():
        from aiogram.fsm.context import FSMContext
        from aiogram.fsm.storage.base import StorageKey
        from aiogram.fsm.storage.memory import MemoryStorage

        storage = MemoryStorage()
        key = StorageKey(bot_id=0, chat_id=1, user_id=1)
        fsm_state = FSMContext(storage=storage, key=key)

        await master.on_project_action(callback, fsm_state)

    asyncio.run(_invoke())

    # 验证：消息应该被编辑（刷新项目列表）
    assert len(callback_message._edits) > 0, "停止全部项目后应该编辑消息（刷新项目列表）"


@pytest.mark.asyncio
async def test_notify_restart_success_pushes_project_overview(repo: ProjectRepository, tmp_path: Path, monkeypatch):
    """重启信号存在时，应该在上线通知后推送项目列表。"""
    manager = _build_manager(repo, tmp_path)
    master.MANAGER = manager

    restart_signal = tmp_path / "state" / "restart_signal.json"
    restart_signal.parent.mkdir(parents=True, exist_ok=True)
    payload = {
        "chat_id": 321,
        "timestamp": datetime.now(master.LOCAL_TZ).isoformat(),
    }
    restart_signal.write_text(json.dumps(payload), encoding="utf-8")

    monkeypatch.setattr(master, "RESTART_SIGNAL_PATH", restart_signal)
    monkeypatch.setattr(master, "LEGACY_RESTART_SIGNAL_PATHS", tuple())

    overview_mock = AsyncMock()
    monkeypatch.setattr(master, "_send_projects_overview_to_chat", overview_mock)
    monkeypatch.setattr(master.asyncio, "sleep", AsyncMock())

    bot = SimpleNamespace(send_message=AsyncMock())

    await master._notify_restart_success(bot)

    overview_mock.assert_awaited_once()
    called_bot, called_chat, _manager = overview_mock.await_args_list[0].args[:3]
    assert called_bot is bot
    assert called_chat == payload["chat_id"]
    assert _manager is manager


@pytest.mark.asyncio
async def test_notify_restart_success_missing_signal_triggers_admin_overview(repo: ProjectRepository, tmp_path: Path, monkeypatch):
    """缺少重启信号但预期重启时，应向管理员推送项目列表。"""
    manager = _build_manager(repo, tmp_path)
    manager.admin_ids = [111, 222]
    master.MANAGER = manager

    monkeypatch.setenv("MASTER_RESTART_EXPECTED", "1")
    monkeypatch.setattr(master, "RESTART_SIGNAL_PATH", tmp_path / "state" / "restart_signal.json")
    monkeypatch.setattr(master, "LEGACY_RESTART_SIGNAL_PATHS", tuple())

    overview_mock = AsyncMock()
    monkeypatch.setattr(master, "_send_projects_overview_to_chat", overview_mock)
    monkeypatch.setattr(master.asyncio, "sleep", AsyncMock())

    bot = SimpleNamespace(send_message=AsyncMock())

    await master._notify_restart_success(bot)

    assert overview_mock.await_count == len(manager.admin_ids)
    called_ids = sorted(call.args[1] for call in overview_mock.await_args_list)
    assert called_ids == sorted(manager.admin_ids)


@pytest.mark.asyncio
async def test_notify_restart_success_invalid_chat_id_fallback(repo: ProjectRepository, tmp_path: Path, monkeypatch):
    """重启信号 chat_id 异常时，应退回管理员推送。"""
    manager = _build_manager(repo, tmp_path)
    manager.admin_ids = [333]
    master.MANAGER = manager

    restart_signal = tmp_path / "state" / "restart_signal.json"
    restart_signal.parent.mkdir(parents=True, exist_ok=True)
    payload = {
        "chat_id": "invalid",
        "timestamp": datetime.now(master.LOCAL_TZ).isoformat(),
    }
    restart_signal.write_text(json.dumps(payload), encoding="utf-8")
    monkeypatch.setattr(master, "RESTART_SIGNAL_PATH", restart_signal)
    monkeypatch.setattr(master, "LEGACY_RESTART_SIGNAL_PATHS", tuple())

    overview_mock = AsyncMock()
    monkeypatch.setattr(master, "_send_projects_overview_to_chat", overview_mock)
    monkeypatch.setattr(master.asyncio, "sleep", AsyncMock())

    bot = SimpleNamespace(send_message=AsyncMock())

    await master._notify_restart_success(bot)

    overview_mock.assert_awaited_once()
    assert overview_mock.await_args_list[0].args[1] == manager.admin_ids[0]


@pytest.mark.asyncio
async def test_notify_restart_success_send_failure_triggers_admin_overview(repo: ProjectRepository, tmp_path: Path, monkeypatch):
    """上线通知发送失败时，应改为向管理员广播项目列表。"""
    manager = _build_manager(repo, tmp_path)
    manager.admin_ids = [444, 555]
    master.MANAGER = manager

    restart_signal = tmp_path / "state" / "restart_signal.json"
    restart_signal.parent.mkdir(parents=True, exist_ok=True)
    payload = {
        "chat_id": 999,
        "timestamp": datetime.now(master.LOCAL_TZ).isoformat(),
    }
    restart_signal.write_text(json.dumps(payload), encoding="utf-8")
    monkeypatch.setattr(master, "RESTART_SIGNAL_PATH", restart_signal)
    monkeypatch.setattr(master, "LEGACY_RESTART_SIGNAL_PATHS", tuple())

    overview_mock = AsyncMock()
    monkeypatch.setattr(master, "_send_projects_overview_to_chat", overview_mock)
    monkeypatch.setattr(master.asyncio, "sleep", AsyncMock())

    bot = SimpleNamespace(send_message=AsyncMock(side_effect=RuntimeError("send failed")))

    await master._notify_restart_success(bot)

    assert overview_mock.await_count == len(manager.admin_ids)
    called_ids = sorted(call.args[1] for call in overview_mock.await_args_list)
    assert called_ids == sorted(manager.admin_ids)


@pytest.mark.asyncio
async def test_send_restart_project_overview_deduplicates_chats(repo: ProjectRepository, tmp_path: Path, monkeypatch):
    """项目列表推送应去重，防止触发人重复收到。"""
    manager = _build_manager(repo, tmp_path)
    master.MANAGER = manager

    overview_mock = AsyncMock()
    monkeypatch.setattr(master, "_send_projects_overview_to_chat", overview_mock)
    monkeypatch.setattr(master.asyncio, "sleep", AsyncMock())

    bot = SimpleNamespace()

    await master._send_restart_project_overview(bot, [1, 1, 2])

    assert overview_mock.await_count == 2
    called_ids = sorted(call.args[1] for call in overview_mock.await_args_list)
    assert called_ids == [1, 2]



"""任务模块常量。"""

TASK_STATUSES = (
    "research",
    "test",
    "done",
)

STATUS_ALIASES = {}

NOTE_TYPES = (
    "research",
    "test",
    "bug",
    "misc",
)

TASK_TYPES = (
    "requirement",
    "defect",
    "task",
    "risk",
)

DEFAULT_PRIORITY = 3
DEFAULT_PAGE_SIZE = 10
